<?php

// $Id: disknode.inc,v 1.1.6.12 2007/11/18 15:15:52 mooffie Exp $

/**
 * Drupal version independed code
 * Functions prefixed with __ are internal functions
 * Functiosn prefixed with _ are called by drupal (through the menu)
 * This is done to prevent a naming collision in the future.
 */

/*******************************************************************************
                                     HOOKS
*******************************************************************************/

/**
 * Implementation of hook_help
 OBSOLETE!!!!!
 */
function __disknode_help($section = "admin/help#disknode") {
  switch ($section) {
    case "admin/help#disknode":
      return t("<p>The diskfile module is used to attach files allready on the system to nodes. It's ideal for hosting downloads.</p>");
    case "admin/modules#description":
      return t("Allows articles with an associated file");
    case 'node/add#disknode':
      return t("Add a new node for a file already on disk. It's ideal for hosting downloads.");
      break;
  }
}

/**
 * Implementation of hook_menu().
 */
function __disknode_menu($may_cache) {
  $items = array();
  if ($may_cache) {
    if (variable_get("disknode_asnode", false))
    {
      $items[] = array(
        'path' => 'node/add/disknode',
        'title' => t('disknode'),
        'access' => user_access('create disknode'));
    }

    $items[] = array(
      'path' => 'admin/settings/disknode',
      'title' => t('Disknode'),
      'description' => t('Change Disknode behavior'),
      'callback' => 'drupal_get_form',
      'callback arguments' => array('disknode_settings'),
      'access' => user_access('administer site configuration'),
      'type' => MENU_NORMAL_ITEM, // optional
    );
 
   $items[] = array(
      'path' => 'disknode/browse',
      'callback' => '_disknode_browse',
      'type' => MENU_CALLBACK,
      'access' => user_access('create disknode'));
    $items[] = array(
      'path' => 'disknode/upload',
      'callback' => '_disknode_upload',
      'type' => MENU_CALLBACK,
      'access' => user_access('upload files'));
    $items[] = array(
      'path' => 'disknode/mkdir',
      'callback' => '_disknode_mkdir',
      'type' => MENU_CALLBACK,
      'access' => user_access('create directory'));
    $items[] = array(
      'path' => 'disknode/rmdir',
      'callback' => '_disknode_rmdir',
      'type' => MENU_CALLBACK,
      'access' => user_access('create directory'));
    $items[] = array(
      'path' => 'disknode/info',
      'callback' => '_disknode_info',
      'type' => MENU_CALLBACK,
      'access' => user_access('create disknode'));
    $items[] = array(
      'path' => 'disknode/convert',
      'callback' => '_disknode_convert',
      'type' => MENU_CALLBACK,
      'access' => user_access('administer site configuration'));
  }
  $items[] = array(
    'path' => 'disknode/get',
    'callback' => '_disknode_get',
    'type' => MENU_CALLBACK,
    'callback arguments' => array(arg(2), arg(3)),
    'access' => user_access('download files')
  );

  return $items;
}

/**
 * Theme a single download link.
 *
 * This function should return an HTML string.
 *
 * param $file
 *   The file object
 * param $solitary
 *   A Boolean; TRUE if disknode is configured to allow a maximum of one
 *   attached file per node --you may use this info to alter the appearance
 *   of such links, if you wish.
 */
function theme_disknode_link($file, $solitary) {
  if ($solitary) {
    // When printing a soliary link, use the format:
    //     Downloads: 56  download (5 KB)
    $link = l(t('download'), "disknode/get/$file->fid/". rawurlencode($file->filename), array(
      'title' => t('Download @file', array('@file' => $file->filename)),
      'rel' => 'nofollow'
    ), 'download');
    $counter = t('Downloads: @counter', array('@counter' => $file->downloads));
    $size    = format_size($file->filesize);
    $link    = "$counter | $link ($size)";
  } else {
    // When printing a link which is one of many, use the format:
    //     file.pdf (56 download | 5 KB)
    $link = l($file->filename, "disknode/get/$file->fid/". rawurlencode($file->filename), array(
      'title' => t('Download @file', array('@file' => $file->filename)),
      'rel' => 'nofollow'
    ), 'download');
    $counter = format_plural($file->downloads, '1 download', '@count downloads');
    $size    = format_size($file->filesize);
    $link    = "$link ($counter | $size)";
  }
  return $link;
}

/**
 * Implementation of hook_link().
 */
function __disknode_link($type, $node = 0, $main = 0) {
  $links = array();

  if ($type == 'node' && !empty($node->disknodefiles)) {
    if (disknode_max_per_node() == 1 && count($node->disknodefiles) == 1) {
      // Why do I use a foreach loop to access the solitary element in this
      // array? Because in the future this array may be indexed by $file->fid
      // (ala Drupal's built-in upload.module), so the file isn't necessarily
      // $node->disknodefiles[0].
      foreach ($node->disknodefiles as $file) {
        $links['disknode_download_link'] = array(
          'html' => TRUE,
          'title' => theme('disknode_link', $file, TRUE),
        );
      }
    }
    else {
      foreach ($node->disknodefiles as $file) {
        $links["disknode_download_link_$file->fid"] = array(
          'html' => TRUE,
          'title' => theme('disknode_link', $file, FALSE),
        );
      }
    }
  }

  return $links;
}

// TODO: document this function.
function disknode_max_per_node() {
  $max = (int)variable_get("disknode_maxfiles", 1);
  if (!$max) $max = 1;
  return $max;
}

/**
 * Implementation of hook_perm().
 */
function __disknode_perm() {
  return array('download files', 'create disknode', 'edit own disknodes', 'upload files', 'create directory', 'delete files', 'escape user directory');
}

/**
 * Implementation of hook_menu().
 */
function __disknode_access($op, $node) {
  global $user;

  switch($op) {
    case 'create':
      return user_access("create disknode");

    case 'delete':
    case 'update':
      return user_access("edit own disknodes") && ($user->uid == $node->uid);
  }
}

/**
 * Implementation of hook_file_download().
 */
function disknode_file_download($file) {
  if (user_access('download files')) {
    $result = db_query("SELECT f.* FROM {files} f WHERE f.filepath = '%s'", $file);
    if ($file = db_fetch_object($result)) {
      if (!empty($file->nid) && !node_access('view', node_load($file->nid))) {
        return -1;
      }
      $name = mime_header_encode($file->filename);
      $type = mime_header_encode($file->filemime);
      $disposition = preg_match(',(\?|&)download(&|$),', request_uri()) ? 'attachment' : 'inline';
      // Serve images and text inline for the browser to display rather than download.
      if (ereg('^(text/|image/)', $file->filemime)) {
        $disposition = 'inline';
      }
      return array(
        'Content-Type: '. $type .'; name='. $name,
        'Content-Length: '. $file->filesize,
        'Content-Disposition: '. $disposition .'; filename='. $name,
        'Cache-Control: private',
      );
    }
  }
}

/**
 * Implementation of hook_insert().
 */
function __disknode_insert($node) {
  foreach ($node->disknodefiles as $file) {
    $fid = db_next_id('{files}_fid');
    db_query("INSERT INTO {files} (fid, nid, filename, filepath, filemime, filesize) VALUES (%d, %d, '%s', '%s', '%s', '%s')", $fid, $node->nid, $file->filename, $file->filepath, $file->filemime, $file->filesize);
    db_query("INSERT INTO {downloads} (fid, counter) VALUES (%d, 0)", $fid);
  }
}

/**
 * Implementation of hook_update().
 */
function __disknode_update($node) {
  foreach ($node->disknodefiles as $file) {
    if ($file->fid == -1) {
      $fid = db_next_id('{files}_fid');
      db_query("INSERT INTO {files} (fid, nid, filename, filepath, filemime, filesize) VALUES (%d, %d, '%s', '%s', '%s', '%s')", $fid, $node->nid, $file->filename, $file->filepath, $file->filemime, $file->filesize);
      db_query("INSERT INTO {downloads} (fid, counter) VALUES (%d, 0)", $fid);
    }
    else {
      db_query("UPDATE {files} SET filename = '%s', filepath = '%s', filemime = '%s', filesize = '%s' WHERE nid = '%d' AND fid = '%d'", $file->filename, $file->filepath, $file->filemime, $file->filesize, $node->nid, $file->fid);
    }
  }
  foreach ($node->old_disknodefiles as $fid) {
    db_query("DELETE FROM {files} WHERE fid=%d", $fid);
    db_query("DELETE FROM {downloads} WHERE fid=%d", $fid);
  }
}

/**
 * Implementation of hook_delete().
 */
function __disknode_delete(&$node) {
  $result = db_query("SELECT f.fid FROM {files} f WHERE nid=%d", $node->nid);
  while ($file = db_fetch_object($result)) {
    db_query("DELETE FROM {downloads} WHERE fid=%d", $file->fid);
  }
  db_query("DELETE FROM {files} WHERE nid=%d", $node->nid);
}

/**
 * Implementation of hook_load().
 */
function __disknode_load(&$node) {
  $orderby = "f.filename";
  switch (intval(variable_get('disknode_sorton', 0))) {
    case 1:
      $orderby = "f.filepath";
      break;
    case 2:
      $orderby = "f.filesize";
      break;
    case 3:
      $orderby = "d.counter";
      break;
    case 4:
      $orderby = "f.fid";
      break;
  }
  if (intval(variable_get('disknode_sortdirection', 0)) == 1) $orderby .= " DESC";

  $result = db_query("SELECT f.fid, f.filename, f.filepath, f.filesize, f.filemime, d.counter FROM {files} f LEFT JOIN {downloads} d ON f.fid = d.fid WHERE nid=%d ORDER BY ".$orderby, $node->nid);

  $node->disknodefiles = array();
  while ($file = db_fetch_object($result)) {
    $entry = new stdClass();
    $entry->fid = $file->fid;
    $entry->filename = $file->filename;
    $entry->filepath = $file->filepath;
    $entry->filesize = $file->filesize;
    $entry->filemime = $file->filemime;
    $entry->downloads = intval($file->counter);
    $node->disknodefiles[] = $entry;
  }
}

/**
 * Implementation of hook_validate(). [4.6]
 * Implementation of hook_submit(). [4.7]
 */
function __disknode_prepareFormData(&$node, $validate=false) {
  if (!isset($node->disknodefiles)) $node->disknodefiles = array();
  $node->old_disknodefiles = array();

  $fidlist = array();
  if ($node->nid) {
    $result = db_query("SELECT f.fid, f.filepath FROM {files} f WHERE nid=%d", $node->nid);
    while ($file = db_fetch_object($result)) {
      $fidlist[$file->filepath] = $file->fid;
      $node->old_disknodefiles[$file->fid] = $file->fid;
    }
  }

  $files = explode("\n", $node->filepaths);
  foreach ($files as $filepath) {
    $file = new stdClass();
    $filepath = trim($filepath);
    if ($filepath == "") continue;
    $fp = file_create_path($filepath);
    if (isset($fidlist[$filepath])) {
      $file->fid = $fidlist[$filepath];
      unset($node->old_disknodefiles[$file->fid]);
    }
    else {
      $file->fid = -1; // pending insert
    }
    $file->filepath = $filepath;
    $file->filename = basename($filepath);
    if (!file_exists($fp)) {
      form_set_error('filepaths', t("File '%name' does not exist.", array("%name" => $filepath)));
      $file->filesize = 0;
      $file->filemime = "application/octet-stream";
    }
    else {
      $file->filesize = @filesize($fp);
      $file->filemime = @mime_content_type($fp);
    }
    if (!$validate) $node->disknodefiles[] = $file;
  }
  $maxfiles = disknode_max_per_node();
  if (($maxfiles > 0) && (count($node->files) > $maxfiles)) {
    form_set_error('filepaths', t("Exceeded maximum number of files (%max) for a single node.", array("%max" => $maxfiles)));
  }
}

/*******************************************************************************
                           DISKNODE CUSTOM FUNCTIONS
*******************************************************************************/

function __disknode_getbasedir()
{
  global $user;
  $basedir = variable_get('disknode_base', '');
  if ((intval(variable_get('disknode_peruserdir', 0)) == 1) && !user_access('escape user directory')) {
    $userdir = preg_replace("#^.*(.)(.)$#", "\\1".DIRECTORY_SEPARATOR."\\2", "00".$user->uid);
    $basedir .= DIRECTORY_SEPARATOR.$userdir.DIRECTORY_SEPARATOR.($user->uid);
  }
  return $basedir;
}

function __disknode_mkdir($dirname)
{
  $mkpath = explode("/", $dirname);
  while (($part = array_shift($mkpath)) || $part === '0') {
    $output .= $part.'/';
    if (!file_check_directory(file_create_path($output), FILE_CREATE_DIRECTORY)) {
      break;
    }
  }
}

function __disknode_sanitize_subdir()
{
  $subdir = $_GET["subdir"];
  $fp = file_create_path();
  $subdir = preg_replace("#^".$fp.DIRECTORY_SEPARATOR."?"."(.*)$#", "\\1", file_create_path($subdir));
  //print_r($fp." - ".$subdir." - ".$_GET["subdir"]." - ".file_create_path($subdir));
  if ($subdir == "." || empty($subdir) || (strcmp($subdir, __disknode_getbasedir()) < 0)) $subdir = __disknode_getbasedir();
  return $subdir;
}

function _disknode_get($fid, $fname="") {
  $fid = intval($fid);
  $result = db_fetch_object(db_query("SELECT f.filepath, d.counter FROM {files} f LEFT JOIN {downloads} d ON f.fid = d.fid WHERE f.fid = %d", $fid));
  if (!$result)
  {
    drupal_not_found();
    return;
  }
  if (is_numeric($result->counter)) {
    db_query("UPDATE {downloads} SET counter = counter + 1 WHERE fid=%d", $fid);
  }
  else {
    db_query("INSERT INTO {downloads} (fid, counter) VALUES (%d, 1)", $fid);
  }
  $url = file_create_url(str_replace("%2F", "/", rawurlencode($result->filepath)));
  if (strstr($url, "?") === false) $url .= "?";
  else $url .= "&";
  $url .= "download";
  header("Location: ".$url);
  exit();
}

function _disknode_browse()
{
  $title = "Browse files";
  $filebase = file_create_path();
  if (!empty($_GET["selection"]))
  {
    $_GET["subdir"] = dirname($_GET["selection"]);
  }
  $subdir = __disknode_sanitize_subdir();

  $dbres = db_query("SELECT * FROM {files} WHERE filepath LIKE '".$subdir."%'");
  while ($entry = db_fetch_object($dbres)) {
    $dbentries[$entry->filepath] = $entry;
  }

  $entries = file_scan_directory(file_create_path($subdir), ".*", array(".", "..", "CVS"), 0, false);
  ksort($entries);
  foreach ($entries as $k => $_)
  {
    $entries[$k]->isdir = is_dir($entries[$k]->filename);
    $entries[$k]->filename = preg_replace("#^(".preg_quote($filebase)."/)#", "", $entries[$k]->filename);
    if (!$entries[$k]->isdir) {
      if (isset($dbentries[$entries[$k]->filename]))
        $entries[$k]->dbentry = $dbentries[$entries[$k]->filename];
      else
        $entries[$k]->dbentry = false;
    }
    else $entries[$k]->dbentry = false;
  }
  include(dirname(__FILE__).DIRECTORY_SEPARATOR."disknode.browse.tpl.inc");
}


define("DISKNODE_OP_VIEW", "view");
define("DISKNODE_OP_MOVE", "move");
define("DISKNODE_OP_COPY", "copy");
define("DISKNODE_OP_DELETE", "delete");

function __disknode_validate_fileaccess($filepath, $operation) {
  if (strcmp($filepath, __disknode_getbasedir()) < 0) {
    drupal_set_message(t("You are not allowed to operate outside the base environment set by the system operator."), 'error');
    return false;
  }
  switch ($operation) {
    case DISKNODE_OP_DELETE:
      if (!user_access("delete files")) {
        drupal_set_message(t("You are not allowed to delete files."), 'error');
        return false;
      }
  }
  return true;
}

function _disknode_info() {
  $title = "File information";
  $subdir = __disknode_sanitize_subdir();

  $file = new StdClass();
  $file->filepath = $_GET["target"];
  $file->filename = basename($file->filepath);  
  $fp = file_create_path($file->filepath);
  $file->filetime = filemtime($fp);
  $file->size = filesize($fp);
  $file->mimetype = mime_content_type($fp);

  if (!__disknode_validate_fileaccess($file->filepath, DISKNODE_OP_VIEW))
  {
    echo theme_status_messages();
    return;
  }

  $file->nodes = array();
  $res = db_query("SELECT n.nid, n.title, f.fid FROM {node} n LEFT JOIN {files} f ON n.nid = f.nid WHERE f.filepath = '%s'", $file->filepath);
  while ($node = db_fetch_object($res)) {
    $node->link = url("node/".$node->nid);
    $file->nodes[] = $node;
  }

  if (isset($_POST['op'])) {
    switch ($_POST['op']) {
      case "move":
        __disknode_info_move($file);
        break;
      case "copy":
        __disknode_info_copy($file);
        break;
      case "delete":
        __disknode_info_delete($file);
        break;
    }
  }

  include(dirname(__FILE__).DIRECTORY_SEPARATOR."disknode.info.tpl.inc");
}

function __disknode_info_move(&$file) {
  if (!__disknode_validate_fileaccess($file->filepath, DISKNODE_OP_MOVE)) return;

  $newname = trim($_POST["newname"]);
  if (strcmp($newname, $file->filepath) == 0) {
    drupal_set_message(t("No change made since the new name is identical to the current name"), 'error');
    return;
  }
  if (!__disknode_validate_fileaccess($newname, DISKNODE_OP_MOVE)) return;

  $source = file_create_path($file->filepath);
  $newname = file_create_path($newname);
  if (!is_dir(dirname($newname))) {
    drupal_set_message(t("Target directory doesn't exist."), 'error');
    return;
  }
  if (file_move($source, $newname, FILE_EXISTS_ERROR)) {
    $source = preg_replace("#^".file_create_path("")."/(.*)$#", "\\1", $source);
    if (intval($_POST["updatenodes"]) == 1) {
      db_query("UPDATE {files} SET filepath = '%s', filename = '%s' WHERE filepath = '%s'", $source, basename($source), $file->filepath);
    }
    $file->filepath = $source;
    drupal_set_message(t("Moved file to %loc", array("%loc" => $file->filepath)));
    drupal_goto("disknode/info", "target=".$file->filepath);
  }
}

function __disknode_info_copy(&$file) {
  if (!__disknode_validate_fileaccess($file->filepath, DISKNODE_OP_COPY)) return;

  $newname = trim($_POST["newname"]);
  if (!__disknode_validate_fileaccess($newname, DISKNODE_OP_COPY)) return;

  $source = file_create_path($file->filepath);
  $newname = file_create_path($newname);
  if (!is_dir(dirname($newname))) {
    drupal_set_message(t("Target directory doesn't exist."), 'error');
    return;
  }
  if (file_copy($source, $newname, FILE_EXISTS_ERROR)) {
    $source = preg_replace("#^".file_create_path("")."/(.*)$#", "\\1", $source);
    drupal_set_message(t("Copied %orig to %new", array("%new" => $source, "%orig" => $file->filepath)));
    drupal_goto("disknode/info", "target=".$source);
  }
}

function __disknode_info_delete(&$file) {
  if (!__disknode_validate_fileaccess($file->filepath, DISKNODE_OP_DELETE)) return;

  if (intval($_POST["yesimsure"]) != 1) {
    drupal_set_message(t("Please check 'I am sure' to delete the file."), 'error');
    return;
  }

  if (intval($_POST["cleannodes"]) == 1) {
    $result = db_query("SELECT f.fid FROM {files} f WHERE filepath = '%s'", $file->filepath);
    while ($res = db_fetch_object($result)) {
      db_query("DELETE FROM {downloads} WHERE fid=%d", $res->fid);
      db_query("DELETE FROM {files} WHERE fid=%d", $res->fid);
    }
  }
  $fp = file_create_path($file->filepath);
  file_delete($fp);
  drupal_set_message(t("File '%name' deleted.", array("%name" => $file->filepath)));
  drupal_goto("disknode/browse", "subdir=".dirname($file->filepath));
}

if (!function_exists("mime_content_type")) {
  function mime_content_type($f) 
  {
    $mime = '';
    if (substr(PHP_OS, 0, 3) != 'WIN') {
      // TODO: might need to be improved
    }
    else {
      $f = escapeshellarg($f);
      $mime = trim( exec("file -bi ".$f) );
    }
    if ($mime && !empty($mime)) return $mime;
    return "application/octet-stream";
  }
}

function __disknode_downloadfile($url){
  define("DOWNLOAD_CHUNK", 8192);

  $result = new StdClass();
  $result->url = $url;
  $result->size = 0;
  $result->error = "";

  $inp = @fopen($url, "rb");
  if (!$inp) {
    $result->error = "Unable to open ".$url;
    drupal_set_message($result->error, 'error');
    return $result;
  }
  $result->filepath = tempnam(file_create_path(variable_get('file_directory_temp', FILE_DIRECTORY_TEMP)), "disknode_");
  $outp = fopen($result->filepath, "wb");
  if (!$outp) {
    fclose($inp);
    $result->error = "Unable to open temporary file ".$result->filepath;
    drupal_set_message($result->error, 'error');
    return $result;
  }
  ignore_user_abort(true);
  set_time_limit(0); // no limit

  while (!feof($inp))
  {
    $result->size += fwrite($outp, fread($inp, DOWNLOAD_CHUNK));
  }
  fclose($inp);
  fclose($outp);
  return $result;
}

function __disknode_do_upload($destdir) {
  $maxsize = intval(variable_get('disknode_maxupload', 0));

  if ($file = file_check_upload('file')) {
    $dest = $destdir."/".__disknode_clean_filename($file->filename);
    __disknode_mkdir($destdir);
    $file = file_save_upload('file', $dest, $_POST["overwrite1"]=="1"?FILE_EXISTS_REPLACE:FILE_EXISTS_ERROR);
    if ($file) {
      drupal_set_message(t("File saved to %path", array("%path" => $file->filepath)), 'status');
    }
    else {
      drupal_set_message(t("Failed to save %dest", array("%dest" => $dest)), 'error');
    }
  }
  else if (isset($_FILES["name"]['file']))
  {
    if (($_FILES["error"]['file'] == 2) || ($_FILES["error"]['file'] == 1))
    {
      drupal_set_message(t('Uploaded file exceeded the maximum size of %size', array('%size' => format_size($maxsize))), 'error');
    }
  }

  if (!empty($_POST["url"])) {
    $file = __disknode_downloadfile($_POST["url"]);
    if (($maxsize > 0) && (filesize($file->filepath) > $maxsize)) {
      drupal_set_message(t('Downloaded file exceeded the maximum size of %size', array('%size' => format_size($maxsize))), 'error');
      file_delete($file->filepath);
    }
    else if (empty($file->error))
    {
      $bname = rawurldecode(preg_replace("#^(.*)(\?(.*))?$#iU", "\\1", basename($_POST["url"])));
      $bname = preg_replace("#^(.*)(\.(php|pl|py|cgi|asp))$#U", "\\1\\2.txt", $bname);

      $dest = $destdir."/".__disknode_clean_filename($bname);
      __disknode_mkdir($destdir);
      if (file_move($file, $dest, $_POST["overwrite2"]=="1"?FILE_EXISTS_REPLACE:FILE_EXISTS_ERROR)) {
        drupal_set_message(t("File saved to %path", array("%path" => $file->filepath)), 'status');
      }
      else {
        drupal_set_message(t("Failed to save %dest", array("%dest" => $dest)), 'error');
      }
    }
    else {
      drupal_set_message("Download error: ".$file->error, 'error');
    }
  }
}

function __disknode_clean_filename($filename) {
  $original = $filename;

  $rename_exts = explode(',', variable_get('disknode_badext', 'php,pl,py,cgi,asp'));
  for ($i = 0; $i < count($rename_exts); $i++) $rename_exts[$i] = trim($rename_exts[$i]);

  // Allow potentially insecure uploads for very savvy users and admin
  if (!variable_get('allow_insecure_uploads', 0)) {
    $parts = explode('.', $filename);
    for ($i = 0; $i < count($parts); $i++) {
      if (empty($parts[$i])) {
        $parts[$i] = '_';
        continue;
      }
      if (in_array(strtolower($parts[$i]), $rename_exts)) {
        $parts[$i] = $parts[$i].'_';
      }
    }
    $filename = implode('.', $parts);
  }

  if ($original != $filename) {
    drupal_set_message(t('Your filename has been renamed to conform to site policy.'));
  }

  return $filename;
}

function __disknode_do_convert() {
  $cnt = 0;
  $res = db_query("SELECT nid, title FROM {node} WHERE type = 'disknode'");
  while ($node = db_fetch_object($res)) {
    drupal_set_message(t('Converting: %node', array("%node" => l($node->title, "node/".$node->nid))));
    db_query("UPDATE {node} SET type = '%s' WHERE nid = '%d'", $_POST["edit"]["target_node"],  $node->nid);
    $cnt++;
  }
  drupal_set_message(t('Converted %cnt disknodes to "%type"', array("%cnt" => $cnt, "%type" => $_POST["edit"]["target_node"])));
  return "";
}
